
(define *bmc-name* "SAL Bounded Model Checker")
(define *bmc-usage* "Usage: sal-bmc [options] <context-name> <assertion-name>")
(define *bmc-examples* "Examples: 
  sal-bmc --depth=10 peterson mutex

  sal-bmc --from=5 --to=10 peterson invalid

  sal-bmc --verbose=3 --depth=10 peterson mutex

  sal-bmc -v 3 --depth=20 peterson mutex

  sal-bmc --depth=10 --assertion=\"(@ mutex peterson)\"

  sal-bmc --depth=20 --assertion=\"(@ at-most-one-ack (arbiter () (10)))\"

  sal-bmc --depth=10 --disable-traceability peterson invalid

  sal-bmc --depth=10 --delta-path peterson invalid

  sal-bmc --depth=10 --lemma=mutex --induction peterson invalid

  sal-bmc --lemma=at-most-one-ack --assertion=\"(@ no-ack-without-request (arbiter () (10)))\"

  sal-bmc --lemma=\"(@ at-most-one-ack (arbiter () (10)))\" --assertion=\"(@ no-ack-without-request (arbiter () (10)))\"")

(front-end/set-categories! '("Help" "Misc" "Assertion" "Pretty Printing" "Code Transformations" "Abstraction" "Traceability" "Path Pretty Printing" "LTL Support" "Verification"))

(sal/load-source! "sal-bmc-core-front-end.scm")

(define (bmc/make-assertion qualified-name-str)
	(make-boolean-assertion qualified-name-str))

(define (bmc/make-lemmas lemmas assertion-qualified-name assertion)
	(sal-bmc/create-assertion-lemmas lemmas assertion-qualified-name assertion))

(define (bmc/invariant assertion)
	(sal-bmc/invariant assertion *from-depth* *to-depth* *acyclic?* (not *iterative?*) *solver-id*))

(define (bmc/liveness assertion)
	(sal-bmc/liveness assertion *to-depth* (not *iterative?*) *solver-id*))

(define (bmc/k-induction assertion lemma-exprs)
	(sal-bmc/k-induction assertion *to-depth* lemma-exprs *solver-id*))

(define (derived-path->original-path counter-example)
	(sal-boolean-flat-module-path->sal-flat-module-path counter-example))

(sal-bmc/main)

