
(sal/load-source! "sal-script-util.scm")

(define *module* #f)
(define *main-module-name* #f)
(define *max-states* 10)

(sal/set-trace-info-enabled! #f)
(sal-smc/enable-counter-examples! #f)

(define (check-module-name-ref help-proc)
	(when (and *module* (or *main-context-name* *main-module-name*))
		(print-error "Invalid combination of arguments, you must not specify context and module names when the option --module is used.")
		(help-proc)
		(exit -1))
	(unless (or *module* (and *main-context-name* *main-module-name*))
		(print-error "A module must be specified.")
		(help-proc)
		(exit -1)))

(define (print-deadlock-states fsm states)
	(print "Total number of deadlock states: " (bdd/num-solutions states (sal-bdd-fsm/num-non-choice-vars fsm)))
	(print "Deadlock states:")
	(sal-bdd-fsm/display-states-without-choices fsm states *max-states* #f))

(front-end/add-simple-option! "Module" "--module=<name>"
															"Qualified module name using the LSAL syntax. Qualified module names are useful to reference modules in parametric contexts. Examples of qualified module names:\n(@ n-arbiter (arbiter () (10)))\n(@ system mutex)"
															(lambda (arg) 
																(set! *module* arg)))

(front-end/add-full-option! 
 "Deadlock Checker" 
 "-u <num>"
 "--max-deadlocks=<num>"
 "Maximum number of deadlock states to be printed (default: 10)."
 (front-end-adapter/nz-nat-arg 
	(lambda (arg) 
		(set! *max-states* arg))))

(front-end/set-categories! '("Help" "Misc" "Module" "Pretty Printing" "Code Transformations" "BDD Interface" "Deadlock Checker"))

(gen-front-end sal-deadlock-checker
							 "SAL Deadlock Checker"
							 "Usage: sal-deadlock-checker [options] <context-name> <module-name>"
"Examples: 
  sal-deadlock-checker peterson system

  sal-deadlock-checker -v 3 peterson system

  sal-deadlock-checker -v 3 -u 5 peterson system

  sal-deadlock-checker --module=\"(@ n-arbiter (arbiter () (10)))\""
(lambda (else)
	(if *main-context-name*
		(if *main-module-name*
			(begin 
				(print-error "Illegal argument `" else "'. Usage:")
				(sal-deadlock-checker/simple-help))
			(set! *main-module-name* else))
		(set! *main-context-name* else)))
(begin
	(check-module-name-ref sal-deadlock-checker/simple-help)
	(sal/set-make-sal-string-reader-proc! make-lsal-string-reader)
	(let* ((module-qualified-name (or *module*
																		(string-append "(@ " *main-module-name* " " *main-context-name* ")")))
				 (fsm (make-bdd-fsm module-qualified-name))
				 (deadlock-states (sal-bdd-fsm/deadlock-states fsm)))
		(cond 
		 ((bdd/false? deadlock-states)
			(print "ok (module does NOT contain deadlock states)."))
		 (else
			(print-deadlock-states fsm deadlock-states))))))

(sal-deadlock-checker/main)
