
(sal/load-source! "sal-script-util.scm")

(define *module* #f)
(define *main-module-name* #f)
(define *depth* 10)
(define *acyclic?* #f)
(define *solver-id* 'ics)

(define (check-module-name-ref help-proc)
	(when (and *module* (or *main-context-name* *main-module-name*))
		(print-error "Invalid combination of arguments, you must not specify context and module names when the option --module is used.")
		(help-proc)
		(exit -1))
	(unless (or *module* (and *main-context-name* *main-module-name*))
		(print-error "A module must be specified.")
		(help-proc)
		(exit -1)))

(front-end/add-simple-option! "Module" "--module=<name>"
															"Qualified module name using the LSAL syntax. Qualified module names are useful to reference modules in parametric contexts. Examples of qualified module names:\n(@ n-arbiter (arbiter () (10)))\n(@ system mutex)"
															(lambda (arg) 
																(set! *module* arg)))
(front-end/add-full-option! 
 "Path Search" 
 "-d <num>"
 "--depth=<num>"
 "Search for a path of length `<num>' (default: 10)."
 (front-end-adapter/nz-nat-arg 
	(lambda (n)
		(set! *depth* n))))

(front-end/add-simple-option! "Path Search" "--acyclic"
															"Consider only acyclic paths."
															(lambda ()
																(set! *acyclic?* #t)))

(front-end/add-full-option! 
 "Path Search" 
 "-s <name>"
 "--solver=<name>"
 "Set sat solver to be used (default: ics)."
 (lambda (arg)
	 (set! *solver-id* (string->symbol arg))))

(front-end/set-categories! '("Help" "Misc" "Module" "Pretty Printing" "Code Transformations" "Abstraction" "Traceability" "Path Pretty Printing" "Path Search"))

(gen-front-end sal-path-finder
							 "SAL Path Finder"
							 "Usage: sal-path-finder [options] <context-name> <module-name>"
"Examples: 
  sal-path-finder peterson system
  
  sal-path-finder --depth=20 four-slot system
  
  sal-path-finder --verbose=3 peterson system
  
  sal-path-finder -v 3 peterson system
  
  sal-path-finder --module=\"(@ n-arbiter (arbiter () (10)))\"
  
  sal-path-finder --disable-traceability peterson system
  
  sal-path-finder --full-trace peterson system"
(lambda (else)
	(if *main-context-name*
		(if *main-module-name*
			(begin 
				(print-error "Illegal argument `" else "'. Usage:")
				(sal-path-finder/simple-help))
			(set! *main-module-name* else))
		(set! *main-context-name* else)))
(begin
	(check-module-name-ref sal-path-finder/simple-help)
	(sal/set-make-sal-string-reader-proc! make-lsal-string-reader)
	(let* ((module-qualified-name (or *module*
																		(string-append "(@ " *main-module-name* " " *main-context-name* ")")))
				 (module (make-boolean-flat-module module-qualified-name)))
		(multiple-value-bind
				(found? path)
				(sal-bmc/find-path module *depth* *acyclic?* *solver-id*)
			(cond
			 ((not found?)
				(print "no path found."))
			 ((eq? path #unspecified)
				(print "path was found, but it couldn't be generated."))
			 (else
				(let ((path (sal-boolean-flat-module-path->sal-flat-module-path path)))
					(print-path path))))))))

(sal-path-finder/main)
						 
							
						 

							